@@ -6,4 +6,7 @@ Camera calibration module.
 from .calibrator import CameraCalibrator
 from .gainselection import GainSelector
 
-__all__ = ["CameraCalibrator", "GainSelector"]
+__all__ = [
+    "CameraCalibrator",
+    "GainSelector",
+]
