@@ -45,6 +45,7 @@ __all__ = [
     "ReconstructedContainer",
     "ReconstructedEnergyContainer",
     "ReconstructedGeometryContainer",
+    "DispContainer",
     "SimulatedCameraContainer",
     "SimulatedShowerContainer",
     "SimulatedShowerDistribution",
@@ -69,6 +70,13 @@ NAN_TIME = Time(0, format="mjd", scale="tai")
 
 #: Used for unsigned integer obs_id or sb_id default values:
 UNKNOWN_ID = np.uint64(np.iinfo(np.uint64).max)
+#: Used for unsigned integer tel_id default value
+UNKNOWN_TEL_ID = np.uint16(np.iinfo(np.uint16).max)
+
+
+obs_id_field = partial(Field, UNKNOWN_ID, description="Observation Block ID")
+event_id_field = partial(Field, UNKNOWN_ID, description="Array Event ID")
+tel_id_field = partial(Field, UNKNOWN_TEL_ID, description="Telescope ID")
 
 
 class SchedulingBlockType(enum.Enum):
@@ -161,8 +169,8 @@ class EventIndexContainer(Container):
     """index columns to include in event lists, common to all data levels"""
 
     default_prefix = ""  # don't want to prefix these
-    obs_id = Field(0, "observation identifier")
-    event_id = Field(0, "event identifier")
+    obs_id = obs_id_field()
+    event_id = event_id_field()
 
 
 class TelEventIndexContainer(Container):
@@ -172,9 +180,9 @@ class TelEventIndexContainer(Container):
     """
 
     default_prefix = ""  # don't want to prefix these
-    obs_id = Field(0, "observation identifier")
-    event_id = Field(0, "event identifier")
-    tel_id = Field(0, "telescope identifier")
+    obs_id = obs_id_field()
+    event_id = event_id_field()
+    tel_id = tel_id_field()
 
 
 class BaseHillasParametersContainer(Container):
@@ -582,7 +590,7 @@ class SimulatedShowerContainer(Container):
         nan * u.g / (u.cm**2), "Simulated Xmax value", unit=u.g / (u.cm**2)
     )
     shower_primary_id = Field(
-        -1,
+        np.int16(np.iinfo(np.int16).max),
         "Simulated shower primary ID 0 (gamma), 1(e-),"
         "2(mu-), 100*A+Z for nucleons and nuclei,"
         "negative for antimatter.",
@@ -747,6 +755,11 @@ class ReconstructedGeometryContainer(Container):
     alt_uncert = Field(nan * u.deg, "reconstructed altitude uncertainty", unit=u.deg)
     az = Field(nan * u.deg, "reconstructed azimuth", unit=u.deg)
     az_uncert = Field(nan * u.deg, "reconstructed azimuth uncertainty", unit=u.deg)
+    ang_distance_uncert = Field(
+        nan * u.deg,
+        "uncertainty radius of reconstructed altitude-azimuth position",
+        unit=u.deg,
+    )
     core_x = Field(
         nan * u.m, "reconstructed x coordinate of the core position", unit=u.m
     )
@@ -823,16 +836,12 @@ class ParticleClassificationContainer(Container):
 
     default_prefix = ""
 
-    # TODO: Do people agree on this? This is very MAGIC-like.
-    # TODO: Perhaps an integer classification to support different classes?
-    # TODO: include an error on the prediction?
     prediction = Field(
         nan,
         (
-            "prediction of the classifier, defined between "
-            "[0,1], where values close to 0 are more "
-            "gamma-like, and values close to 1 more "
-            "hadron-like"
+            " prediction of the classifier, defined between [0,1]"
+            ", where values close to 1 mean that the positive class"
+            " (e.g. gamma in gamma-ray analysis) is more likely"
         ),
     )
     is_valid = Field(False, "true if classification parameters are valid")
@@ -840,6 +849,17 @@ class ParticleClassificationContainer(Container):
     telescopes = Field(None, "Telescopes used if stereo, or None if Mono")
 
 
+class DispContainer(Container):
+    """
+    Standard output of disp reconstruction algorithms for origin reconstruction
+    """
+
+    default_prefix = "disp_parameter"
+
+    norm = Field(nan * u.deg, "reconstructed value for disp", unit=u.deg)
+    is_valid = Field(False, "true if the predictions are valid")
+
+
 class ReconstructedContainer(Container):
     """Reconstructed shower info from multiple algorithms"""
 
@@ -872,6 +892,10 @@ class TelescopeReconstructedContainer(ReconstructedContainer):
         default_factory=partial(Map, TelescopeImpactParameterContainer),
         description="map of algorithm to impact parameter info",
     )
+    disp = Field(
+        default_factory=partial(Map, DispContainer),
+        description="map of algorithm to reconstructed disp parameters",
+    )
 
 
 class DL2Container(Container):
@@ -937,10 +961,14 @@ class EventCalibrationContainer(Container):
 
 
 class MuonRingContainer(Container):
-    """Container for the result of a ring fit, center_x, center_y"""
+    """Container for the result of a ring fit in telescope frame"""
 
-    center_x = Field(nan * u.deg, "center (x) of the fitted muon ring", unit=u.deg)
-    center_y = Field(nan * u.deg, "center (y) of the fitted muon ring", unit=u.deg)
+    center_fov_lon = Field(
+        nan * u.deg, "center (fov_lon) of the fitted muon ring", unit=u.deg
+    )
+    center_fov_lat = Field(
+        nan * u.deg, "center (fov_lat) of the fitted muon ring", unit=u.deg
+    )
     radius = Field(nan * u.deg, "radius of the fitted muon ring", unit=u.deg)
     center_phi = Field(
         nan * u.deg, "Angle of ring center within camera plane", unit=u.deg
@@ -968,7 +996,31 @@ class MuonParametersContainer(Container):
     )
     intensity_ratio = Field(nan, "Intensity ratio of pixels in the ring to all pixels")
     mean_squared_error = Field(
-        nan, "MSE of the deviation of all pixels after cleaning from the ring fit"
+        nan * u.deg**2,
+        "MSE of the deviation of all pixels after cleaning from the ring fit",
+    )
+
+
+class MuonTelescopeContainer(Container):
+    """
+    Container for muon analysis
+    """
+
+    ring = Field(default_factory=MuonRingContainer, description="muon ring fit")
+    parameters = Field(
+        default_factory=MuonParametersContainer, description="muon parameters"
+    )
+    efficiency = Field(
+        default_factory=MuonEfficiencyContainer, description="muon efficiency"
+    )
+
+
+class MuonContainer(Container):
+    """Root container for muon parameters"""
+
+    tel = Field(
+        default_factory=partial(Map, MuonTelescopeContainer),
+        description="map of tel_id to MuonTelescopeContainer",
     )
 
 
@@ -1158,7 +1210,7 @@ class SimulatedShowerDistribution(Container):
 
     default_prefix = ""
 
-    obs_id = Field(-1, description="links to which events this corresponds to")
+    obs_id = obs_id_field()
     hist_id = Field(-1, description="Histogram ID")
     n_entries = Field(-1, description="Number of entries in the histogram")
     bins_energy = Field(
@@ -1208,6 +1260,9 @@ class ArrayEventContainer(Container):
         default_factory=MonitoringContainer,
         description="container for event-wise monitoring data (MON)",
     )
+    muon = Field(
+        default_factory=MuonContainer, description="Container for muon analysis results"
+    )
 
 
 class SchedulingBlockContainer(Container):
@@ -1243,7 +1298,7 @@ class ObservationBlockContainer(Container):
     """Stores information about the observation"""
 
     default_prefix = ""
-    obs_id = Field(UNKNOWN_ID, "Observation Block ID", type=np.uint64)
+    obs_id = obs_id_field()
     sb_id = Field(UNKNOWN_ID, "ID of the parent SchedulingBlock", type=np.uint64)
     producer_id = Field(
         "unknown",
