@@ -1,14 +1,13 @@
-from .fitting import kundu_chaudhuri_circle_fit
 from .features import (
-    mean_squared_error,
     intensity_ratio_inside_ring,
+    mean_squared_error,
     ring_completeness,
     ring_containment,
 )
-
-from .ring_fitter import MuonRingFitter
+from .fitting import kundu_chaudhuri_circle_fit
 from .intensity_fitter import MuonIntensityFitter
-
+from .processor import MuonProcessor
+from .ring_fitter import MuonRingFitter
 
 __all__ = [
     "MuonIntensityFitter",
@@ -18,4 +17,5 @@ __all__ = [
     "intensity_ratio_inside_ring",
     "ring_completeness",
     "ring_containment",
+    "MuonProcessor",
 ]
