@@ -1,12 +1,15 @@
-from ctapipe.image.tests.test_hillas import create_sample_image
-from ctapipe.image.hillas import hillas_parameters
-from ctapipe.image.concentration import concentration_parameters
 import astropy.units as u
 import pytest
 
+from ctapipe.image.concentration import concentration_parameters
+from ctapipe.image.hillas import hillas_parameters
+from ctapipe.image.tests.test_hillas import create_sample_image
 
-def test_concentration():
-    geom, image, clean_mask = create_sample_image("30d")
+
+def test_concentration(prod5_lst):
+
+    geom = prod5_lst.camera.geometry
+    image, clean_mask = create_sample_image(psi="30d", geometry=geom)
 
     hillas = hillas_parameters(geom[clean_mask], image[clean_mask])
 
@@ -18,8 +21,9 @@ def test_concentration():
 
 
 @pytest.mark.filterwarnings("error")
-def test_width_0():
-    geom, image, clean_mask = create_sample_image("30d")
+def test_width_0(prod5_lst):
+    geom = prod5_lst.camera.geometry
+    image, clean_mask = create_sample_image(psi="30d", geometry=geom)
 
     hillas = hillas_parameters(geom[clean_mask], image[clean_mask])
     hillas.width = 0 * u.m
@@ -28,19 +32,16 @@ def test_width_0():
     assert conc.core == 0
 
 
-def test_no_pixels_near_cog():
-    geom, image, clean_mask = create_sample_image("30d")
+def test_no_pixels_near_cog(prod5_lst):
+    geom = prod5_lst.camera.geometry
+    image, clean_mask = create_sample_image(psi="30d", geometry=geom)
 
     hillas = hillas_parameters(geom[clean_mask], image[clean_mask])
 
     # remove pixels close to cog from the cleaning pixels
     clean_mask &= ((geom.pix_x - hillas.x) ** 2 + (geom.pix_y - hillas.y) ** 2) >= (
-        2 * geom.pixel_width ** 2
+        2 * geom.pixel_width**2
     )
 
     conc = concentration_parameters(geom[clean_mask], image[clean_mask], hillas)
     assert conc.cog == 0
-
-
-if __name__ == "__main__":
-    test_concentration()
