@@ -5,11 +5,12 @@ from pytest import raises
 def test_guessing():
     from ctapipe.instrument import guess_telescope
 
-    guess = guess_telescope(2048, 2.28)
+    # n_tiles should not be used for GCT since pixels + focal length is unique
+    guess = guess_telescope(2048, 2.28, None)
     assert guess.type == "SST"
     assert guess.name == "GCT"
 
-    guess = guess_telescope(2048, 2.28 * u.m)
+    guess = guess_telescope(2048, 2.28 * u.m, 32)
     assert guess.type == "SST"
     assert guess.name == "GCT"
 
@@ -23,6 +24,9 @@ def test_guessing():
     assert guess.camera_name == "FlashCam"
     assert guess.type == "MST"
 
+    assert guess_telescope(1039, 16.97, 964).name == "MAGIC-1"
+    assert guess_telescope(1039, 16.97, 247).name == "MAGIC-2"
+
 
 def test_unknown_telescope():
     from ctapipe.instrument.guess import unknown_telescope
@@ -37,7 +41,7 @@ def test_unknown_telescope():
     assert tel.name == "UNKNOWN-10M2"
     assert tel.camera_name == "UNKNOWN-2048PX"
 
-    tel = unknown_telescope(100 * u.m ** 2, 2048)
+    tel = unknown_telescope(100 * u.m**2, 2048)
     assert tel.type == "MST"
     assert tel.name == "UNKNOWN-100M2"
     assert tel.camera_name == "UNKNOWN-2048PX"
