@@ -1,25 +1,28 @@
-from .eventseeker import EventSeeker
-from .eventsource import EventSource
-from .hdf5tableio import HDF5TableReader, HDF5TableWriter
-from .tableio import TableWriter, TableReader
-from .tableloader import TableLoader
-from .datalevels import DataLevel
+"""
+ctapipe io module
+
+# order matters to prevent circular imports
+isort:skip_file
+"""
 from .astropy_helpers import read_table, write_table
-from .datawriter import DataWriter, DATA_MODEL_VERSION
+from .datalevels import DataLevel
+from .eventsource import EventSource
+from .eventseeker import EventSeeker
+from .tableio import TableReader, TableWriter
+from .hdf5tableio import HDF5TableReader, HDF5TableWriter
+from .tableloader import TableLoader
+from .hdf5merger import HDF5Merger
 
-from ..core.plugins import detect_and_import_io_plugins
-
-# import event sources to make them visible to EventSource.from_url
-from .simteleventsource import SimTelEventSource
 from .hdf5eventsource import HDF5EventSource, get_hdf5_datalevels
+from .simteleventsource import SimTelEventSource
 
-# import IO plugins with their event sources
-detect_and_import_io_plugins()
+from .datawriter import DATA_MODEL_VERSION, DataWriter
 
 
 __all__ = [
     "HDF5TableWriter",
     "HDF5TableReader",
+    "HDF5Merger",
     "TableWriter",
     "TableReader",
     "TableLoader",
