@@ -109,9 +109,13 @@ def write_reference_metadata_headers(
     instrument_info: meta.Instrument
         instrument metadata
     """
-    activity = PROV.current_activity.provenance
-    category = "Sim" if is_simulation else "Other"
+    activity = PROV.current_activity
+    if activity is None and len(PROV.finished_activities) > 0:
+        # assume that we write provenance for a "just finished activity"
+        activity = PROV.finished_activities[-1]
 
+    activity_meta = meta.Activity.from_provenance(activity.provenance)
+    category = "Sim" if is_simulation else "Other"
     reference = meta.Reference(
         contact=contact_info,
         product=meta.Product(
@@ -129,7 +133,7 @@ def write_reference_metadata_headers(
             subtype="",
             id_=",".join(str(x) for x in obs_ids),
         ),
-        activity=meta.Activity.from_provenance(activity),
+        activity=activity_meta,
         instrument=instrument_info,
     )
 
