@@ -8,6 +8,7 @@ import numpy as np
 import tables
 from astropy.utils.decorators import lazyproperty
 
+from ctapipe.atmosphere import AtmosphereDensityProfile
 from ctapipe.instrument.optics import FocalLengthKind
 
 from ..containers import (
@@ -22,6 +23,10 @@ from ..containers import (
     IntensityStatisticsContainer,
     LeakageContainer,
     MorphologyContainer,
+    MuonEfficiencyContainer,
+    MuonParametersContainer,
+    MuonRingContainer,
+    MuonTelescopeContainer,
     ObservationBlockContainer,
     ParticleClassificationContainer,
     PeakTimeStatisticsContainer,
@@ -42,6 +47,7 @@ from ..core import Container, Field
 from ..core.traits import UseEnum
 from ..instrument import SubarrayDescription
 from ..utils import IndexFinder
+from .astropy_helpers import read_table
 from .datalevels import DataLevel
 from .eventsource import EventSource
 from .hdf5tableio import HDF5TableReader, get_column_attrs
@@ -63,6 +69,7 @@ DL2_CONTAINERS = {
 
 COMPATIBLE_DATA_MODEL_VERSIONS = [
     "v4.0.0",
+    "v5.0.0",
 ]
 
 
@@ -79,12 +86,42 @@ def get_hdf5_datalevels(h5file):
     if "/dl1/event/telescope/parameters" in h5file.root:
         datalevels.append(DataLevel.DL1_PARAMETERS)
 
+    if "/dl1/event/telescope/muon" in h5file.root:
+        datalevels.append(DataLevel.DL1_MUON)
+
     if "/dl2" in h5file.root:
         datalevels.append(DataLevel.DL2)
 
     return tuple(datalevels)
 
 
+def read_atmosphere_density_profile(
+    h5file: tables.File, path="/simulation/service/atmosphere_density_profile"
+):
+    """return a subclass of AtmosphereDensityProfile by
+    reading a table in a h5 file
+
+    Parameters
+    ----------
+    h5file: tables.File
+        file handle of HDF5 file
+    path: str
+        path in the file where the serialized model is stored as an
+        astropy.table.Table
+
+    Returns
+    -------
+    AtmosphereDensityProfile:
+        subclass depending on type of table
+    """
+
+    if path not in h5file:
+        return None
+
+    table = read_table(h5file=h5file, path=path)
+    return AtmosphereDensityProfile.from_table(table)
+
+
 class HDF5EventSource(EventSource):
     """
     Event source for files in the ctapipe DL1 format.
@@ -128,10 +165,11 @@ class HDF5EventSource(EventSource):
         default_value=FocalLengthKind.EFFECTIVE,
         help=(
             "If both nominal and effective focal lengths are available, "
-            " which one to use for the `CameraFrame` attached"
-            " to the `CameraGeometry` instances in the `SubarrayDescription`"
-            ", which will be used in CameraFrame to TelescopeFrame coordinate"
-            " transforms. The 'nominal' focal length is the one used during "
+            " which one to use for the `~ctapipe.coordinates.CameraFrame` attached"
+            " to the `~ctapipe.instrument.CameraGeometry` instances in the"
+            " `~ctapipe.instrument.SubarrayDescription` which will be used in"
+            " CameraFrame to TelescopeFrame coordinate transforms."
+            " The 'nominal' focal length is the one used during "
             " the simulation, the 'effective' focal length is computed using specialized "
             " ray-tracing from a point light source"
         ),
@@ -170,9 +208,12 @@ class HDF5EventSource(EventSource):
             self._scheduling_block,
             self._observation_block,
         ) = self._parse_sb_and_ob_configs()
-        self.datamodel_version = self.file_.root._v_attrs[
-            "CTA PRODUCT DATA MODEL VERSION"
-        ]
+
+        version = self.file_.root._v_attrs["CTA PRODUCT DATA MODEL VERSION"]
+        self.datamodel_version = tuple(map(int, version.lstrip("v").split(".")))
+        self._obs_ids = tuple(
+            self.file_.root.configuration.observation.observation_block.col("obs_id")
+        )
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.close()
@@ -214,8 +255,16 @@ class HDF5EventSource(EventSource):
                 return False
 
             # we can now read both R1 and DL1
+            has_muons = "/dl1/event/telescope/muon" in f.root
             datalevels = set(metadata["CTA PRODUCT DATA LEVELS"].split(","))
-            if not datalevels.intersection(("R1", "DL1_IMAGES", "DL1_PARAMETERS")):
+            datalevels = datalevels & {
+                "R1",
+                "DL1_IMAGES",
+                "DL1_PARAMETERS",
+                "DL2",
+                "DL1_MUON",
+            }
+            if not datalevels and not has_muons:
                 return False
 
         return True
@@ -237,6 +286,13 @@ class HDF5EventSource(EventSource):
                 return True
         return False
 
+    @lazyproperty
+    def has_muon_parameters(self):
+        """
+        True for files that contain muon parameters
+        """
+        return "/dl1/event/telescope/muon" in self.file_.root
+
     @property
     def subarray(self):
         return self._subarray
@@ -246,8 +302,12 @@ class HDF5EventSource(EventSource):
         return get_hdf5_datalevels(self.file_)
 
     @lazyproperty
+    def atmosphere_density_profile(self) -> AtmosphereDensityProfile:
+        return read_atmosphere_density_profile(self.file_)
+
+    @property
     def obs_ids(self):
-        return list(np.unique(self.file_.root.dl1.event.subarray.trigger.col("obs_id")))
+        return self._obs_ids
 
     @property
     def scheduling_blocks(self) -> Dict[int, SchedulingBlockContainer]:
@@ -314,6 +374,18 @@ class HDF5EventSource(EventSource):
 
         return scheduling_blocks, observation_blocks
 
+    def _is_hillas_in_camera_frame(self):
+        parameters_group = self.file_.root.dl1.event.telescope.parameters
+        telescope_tables = parameters_group._v_children.values()
+
+        # in case of no parameters, it doesn't matter, we just return False
+        if len(telescope_tables) == 0:
+            return False
+
+        # check the first telescope table
+        one_telescope = parameters_group._v_children.values()[0]
+        return "camera_frame_hillas_intensity" in one_telescope.colnames
+
     def _generator(self):
         """
         Yield ArrayEventContainer to iterate through events.
@@ -352,13 +424,16 @@ class HDF5EventSource(EventSource):
                 }
 
         if DataLevel.DL1_PARAMETERS in self.datalevels:
-            # FIXME: check units or config, not version. We have a switch.
-            if self.datamodel_version >= "v2.1.0":
-                hillas_cls = HillasParametersContainer
-                timing_cls = TimingParametersContainer
-            else:
+            hillas_cls = HillasParametersContainer
+            timing_cls = TimingParametersContainer
+            hillas_prefix = "hillas"
+            timing_prefix = "timing"
+
+            if self._is_hillas_in_camera_frame():
                 hillas_cls = CameraHillasParametersContainer
                 timing_cls = CameraTimingParametersContainer
+                hillas_prefix = "camera_frame_hillas"
+                timing_prefix = "camera_frame_timing"
 
             param_readers = {
                 table.name: self.reader.read(
@@ -373,8 +448,8 @@ class HDF5EventSource(EventSource):
                         PeakTimeStatisticsContainer,
                     ),
                     prefixes=[
-                        "hillas",
-                        "timing",
+                        hillas_prefix,
+                        timing_prefix,
                         "leakage",
                         "concentration",
                         "morphology",
@@ -396,7 +471,7 @@ class HDF5EventSource(EventSource):
                             IntensityStatisticsContainer,
                         ],
                         prefixes=[
-                            "true_hillas",
+                            f"true_{hillas_prefix}",
                             "true_leakage",
                             "true_concentration",
                             "true_morphology",
@@ -406,6 +481,19 @@ class HDF5EventSource(EventSource):
                     for table in self.file_.root.dl1.event.telescope.parameters
                 }
 
+        if self.has_muon_parameters:
+            muon_readers = {
+                table.name: self.reader.read(
+                    f"/dl1/event/telescope/muon/{table.name}",
+                    containers=[
+                        MuonRingContainer,
+                        MuonParametersContainer,
+                        MuonEfficiencyContainer,
+                    ],
+                )
+                for table in self.file_.root.dl1.event.telescope.muon
+            }
+
         dl2_readers = {}
         if DL2_SUBARRAY_GROUP in self.file_.root:
             dl2_group = self.file_.root[DL2_SUBARRAY_GROUP]
@@ -444,7 +532,6 @@ class HDF5EventSource(EventSource):
                         key: HDF5TableReader(self.file_).read(
                             table._v_pathname,
                             containers=container,
-                            prefixes=(f"{algorithm}_tel",),
                         )
                         for key, table in algorithm_group._v_children.items()
                     }
@@ -522,18 +609,26 @@ class HDF5EventSource(EventSource):
 
                 data.trigger.tel[tel_index.tel_id] = tel_trigger
 
+            if self.is_simulation:
+                data.simulation.shower = next(mc_shower_reader)
+
+            for kind, readers in dl2_readers.items():
+                c = getattr(data.dl2.stereo, kind)
+                for algorithm, reader in readers.items():
+                    c[algorithm] = next(reader)
+
             # this needs to stay *after* reading the telescope trigger table
+            # and after reading all subarray event information, so that we don't
+            # go out of sync
             if len(data.trigger.tels_with_trigger) == 0:
                 continue
 
             self._fill_array_pointing(data, array_pointing_finder)
             self._fill_telescope_pointing(data, tel_pointing_finder)
 
-            if self.is_simulation:
-                data.simulation.shower = next(mc_shower_reader)
-
             for tel_id in data.trigger.tel.keys():
                 key = f"tel_{tel_id:03d}"
+
                 if self.allowed_tels and tel_id not in self.allowed_tels:
                     continue
 
@@ -547,33 +642,13 @@ class HDF5EventSource(EventSource):
                     simulated = data.simulation.tel[tel_id]
 
                 if DataLevel.DL1_IMAGES in self.datalevels:
-                    if key not in image_readers:
-                        logger.debug(
-                            f"Triggered telescope {tel_id} is missing "
-                            "from the image table."
-                        )
-                        continue
-
                     data.dl1.tel[tel_id] = next(image_readers[key])
 
                     if self.has_simulated_dl1:
-                        if key not in simulated_image_iterators:
-                            logger.warning(
-                                f"Triggered telescope {tel_id} is missing "
-                                "from the simulated image table, but was present at the "
-                                "reconstructed image table."
-                            )
-                            continue
                         simulated_image_row = next(simulated_image_iterators[key])
                         simulated.true_image = simulated_image_row["true_image"]
 
                 if DataLevel.DL1_PARAMETERS in self.datalevels:
-                    if f"tel_{tel_id:03d}" not in param_readers:
-                        logger.debug(
-                            f"Triggered telescope {tel_id} is missing "
-                            "from the parameters table."
-                        )
-                        continue
                     # Is there a smarter way to unpack this?
                     # Best would probbaly be if we could directly read
                     # into the ImageParametersContainer
@@ -588,16 +663,7 @@ class HDF5EventSource(EventSource):
                         peak_time_statistics=params[6],
                     )
                     if self.has_simulated_dl1:
-                        if f"tel_{tel_id:03d}" not in simulated_param_readers:
-                            logger.debug(
-                                f"Triggered telescope {tel_id} is missing "
-                                "from the simulated parameters table, but was "
-                                "present at the reconstructed parameters table."
-                            )
-                            continue
-                        simulated_params = next(
-                            simulated_param_readers[f"tel_{tel_id:03d}"]
-                        )
+                        simulated_params = next(simulated_param_readers[key])
                         simulated.true_parameters = ImageParametersContainer(
                             hillas=simulated_params[0],
                             leakage=simulated_params[1],
@@ -606,15 +672,23 @@ class HDF5EventSource(EventSource):
                             intensity_statistics=simulated_params[4],
                         )
 
+                if self.has_muon_parameters:
+                    ring, parameters, efficiency = next(muon_readers[key])
+                    data.muon.tel[tel_id] = MuonTelescopeContainer(
+                        ring=ring,
+                        parameters=parameters,
+                        efficiency=efficiency,
+                    )
+
                 for kind, algorithms in dl2_tel_readers.items():
                     c = getattr(data.dl2.tel[tel_id], kind)
                     for algorithm, readers in algorithms.items():
                         c[algorithm] = next(readers[key])
 
-            for kind, readers in dl2_readers.items():
-                c = getattr(data.dl2.stereo, kind)
-                for algorithm, reader in readers.items():
-                    c[algorithm] = next(reader)
+                        # change prefix to new data model
+                        if kind == "impact" and self.datamodel_version == (4, 0, 0):
+                            prefix = f"{algorithm}_tel_{c[algorithm].default_prefix}"
+                            c[algorithm].prefix = prefix
 
             yield data
             counter += 1
@@ -663,11 +737,13 @@ class HDF5EventSource(EventSource):
         # Same comments as to _fill_array_pointing apply
         pointing_group = self.file_.root.dl1.monitoring.telescope.pointing
         for tel_id in data.trigger.tel.keys():
+            key = f"tel_{tel_id:03d}"
+
             if self.allowed_tels and tel_id not in self.allowed_tels:
                 continue
 
-            tel_pointing_table = pointing_group[f"tel_{tel_id:03d}"]
-            closest_time_index = tel_pointing_finder[f"tel_{tel_id:03d}"].closest(
+            tel_pointing_table = pointing_group[key]
+            closest_time_index = tel_pointing_finder[key].closest(
                 data.trigger.tel[tel_id].time.mjd
             )
 
