@@ -6,52 +6,74 @@ from pathlib import Path
 
 import numpy as np
 import tables
-from traitlets.config import Config
 from astropy import units as u
+from traitlets.config import Config
+
 from ctapipe.calib import CameraCalibrator
+from ctapipe.containers import (
+    ParticleClassificationContainer,
+    ReconstructedEnergyContainer,
+    ReconstructedGeometryContainer,
+)
 from ctapipe.instrument import SubarrayDescription
 from ctapipe.io import DataLevel, EventSource
 from ctapipe.io.datawriter import DATA_MODEL_VERSION, DataWriter
+from ctapipe.io.hdf5tableio import get_column_attrs
 from ctapipe.utils import get_dataset_path
 
 
 def generate_dummy_dl2(event):
-    """ generate some dummy DL2 info and see if we can write it """
+    """generate some dummy DL2 info and see if we can write it"""
 
     algos = ["HillasReconstructor", "ImPACTReconstructor"]
 
     for algo in algos:
         for tel_id in event.dl1.tel:
-            event.dl2.tel[tel_id].geometry[algo].alt = 70 * u.deg
-            event.dl2.tel[tel_id].geometry[algo].az = 120 * u.deg
-            event.dl2.tel[tel_id].energy[algo].energy = 10 * u.TeV
-            event.dl2.tel[tel_id].classification[algo].prediction = 0.9
+            event.dl2.tel[tel_id].geometry[algo] = ReconstructedGeometryContainer(
+                alt=70 * u.deg,
+                az=120 * u.deg,
+                prefix=f"{algo}_tel",
+            )
 
-        event.dl2.stereo.geometry[algo].alt = 72 * u.deg
-        event.dl2.stereo.geometry[algo].az = 121 * u.deg
-        event.dl2.stereo.geometry[algo].tel_ids = [1, 2, 4]
-        event.dl2.stereo.energy[algo].tel_ids = [1, 2, 4]
-        event.dl2.stereo.energy[algo].energy = 10 * u.TeV
-        event.dl2.stereo.classification[algo].prediction = 0.9
-        event.dl2.stereo.classification[algo].tel_ids = [1, 2, 4]
+            event.dl2.tel[tel_id].energy[algo] = ReconstructedEnergyContainer(
+                energy=10 * u.TeV,
+                prefix=f"{algo}_tel",
+            )
+            event.dl2.tel[tel_id].classification[
+                algo
+            ] = ParticleClassificationContainer(
+                prediction=0.9,
+                prefix=f"{algo}_tel",
+            )
+
+        event.dl2.stereo.geometry[algo] = ReconstructedGeometryContainer(
+            alt=72 * u.deg,
+            az=121 * u.deg,
+            telescopes=[1, 2, 4],
+            prefix=algo,
+        )
+
+        event.dl2.stereo.energy[algo] = ReconstructedEnergyContainer(
+            energy=10 * u.TeV,
+            telescopes=[1, 2, 4],
+            prefix=algo,
+        )
+        event.dl2.stereo.classification[algo] = ParticleClassificationContainer(
+            prediction=0.9,
+            telescopes=[1, 2, 4],
+            prefix=algo,
+        )
 
 
 def test_write(tmpdir: Path):
     """
     Check that we can write and read data from R0-DL2 to files
-
-    Parameters
-    ----------
-    tmpdir :
-        temp directory fixture
     """
 
     output_path = Path(tmpdir / "events.dl1.h5")
     source = EventSource(
-        get_dataset_path("gamma_LaPalma_baseline_20Zd_180Az_prod3b_test.simtel.gz"),
-        max_events=20,
-        allowed_tels=[1, 2, 3, 4],
-        focal_length_choice='nominal',
+        get_dataset_path("gamma_prod5.simtel.zst"),
+        focal_length_choice="EQUIVALENT",
     )
     calibrate = CameraCalibrator(subarray=source.subarray)
 
@@ -60,8 +82,7 @@ def test_write(tmpdir: Path):
         output_path=output_path,
         write_parameters=False,
         write_images=True,
-        write_stereo_shower=True,
-        write_mono_shower=True,
+        write_showers=True,
         write_raw_waveforms=True,
         write_waveforms=True,
     ) as writer:
@@ -76,21 +97,21 @@ def test_write(tmpdir: Path):
 
     # check we can get the subarray description:
     sub = SubarrayDescription.from_hdf(output_path)
-    assert sub.num_tels > 0
+    assert sub.n_tels > 0
 
     # check a few things in the output just to make sure there is output. For a
     # full test of the data model, a verify tool should be created.
     with tables.open_file(output_path) as h5file:
         # check R0:
-        r0tel = h5file.get_node("/r0/event/telescope/tel_001")
+        r0tel = h5file.get_node("/r0/event/telescope/tel_004")
         assert r0tel.col("waveform").max() > 0
 
         # check R1:
-        r1tel = h5file.get_node("/r1/event/telescope/tel_001")
+        r1tel = h5file.get_node("/r1/event/telescope/tel_004")
         assert r1tel.col("waveform").max() > 0
 
         # check DL1:
-        images = h5file.get_node("/dl1/event/telescope/images/tel_001")
+        images = h5file.get_node("/dl1/event/telescope/images/tel_004")
         assert images.col("image").max() > 0.0
         assert (
             h5file.root._v_attrs[
@@ -99,22 +120,22 @@ def test_write(tmpdir: Path):
             == DATA_MODEL_VERSION
         )
         shower = h5file.get_node("/simulation/event/subarray/shower")
+        shower_attrs = get_column_attrs(shower)
         assert len(shower) > 0
         assert shower.col("true_alt").mean() > 0.0
-        assert (
-            shower._v_attrs["true_alt_UNIT"] == "deg"
-        )  # pylint: disable=protected-access
+        assert shower_attrs["true_alt"]["UNIT"] == "deg"
 
-        # check DL2:
-        dl2_energy = h5file.get_node("/dl2/event/subarray/energy/ImPACTReconstructor")
-        assert np.allclose(dl2_energy.col("energy"), 10)
-        assert np.count_nonzero(dl2_energy.col("tel_ids")[0]) == 3
+        # check DL2
+        for prefix in ("ImPACTReconstructor", "HillasReconstructor"):
+            dl2_energy = h5file.get_node(f"/dl2/event/subarray/energy/{prefix}")
+            assert np.allclose(dl2_energy.col(f"{prefix}_energy"), 10)
+            assert np.count_nonzero(dl2_energy.col(f"{prefix}_telescopes")[0]) == 3
 
-        dl2_tel_energy = h5file.get_node(
-            "/dl2/event/telescope/energy/HillasReconstructor/tel_002"
-        )
-        assert np.allclose(dl2_tel_energy.col("energy"), 10)
-        assert "tel_ids" not in dl2_tel_energy
+            dl2_tel_energy = h5file.get_node(
+                f"/dl2/event/telescope/energy/{prefix}/tel_004"
+            )
+            assert np.allclose(dl2_tel_energy.col(f"{prefix}_tel_energy"), 10)
+            assert "telescopes" not in dl2_tel_energy
 
 
 def test_roundtrip(tmpdir: Path):
@@ -129,10 +150,8 @@ def test_roundtrip(tmpdir: Path):
 
     output_path = Path(tmpdir / "events.DL1DL2.h5")
     source = EventSource(
-        get_dataset_path("gamma_LaPalma_baseline_20Zd_180Az_prod3b_test.simtel.gz"),
-        max_events=20,
-        allowed_tels=[1, 2, 3, 4],
-        focal_length_choice='nominal',
+        get_dataset_path("gamma_prod5.simtel.zst"),
+        focal_length_choice="EQUIVALENT",
     )
     calibrate = CameraCalibrator(subarray=source.subarray)
 
@@ -149,8 +168,7 @@ def test_roundtrip(tmpdir: Path):
         transform_peak_time=True,
         peak_time_dtype="int16",
         peak_time_scale=100,
-        write_stereo_shower=True,
-        write_mono_shower=True,
+        write_showers=True,
     ) as write:
         write.log.level = logging.DEBUG
         for event in source:
@@ -167,12 +185,12 @@ def test_roundtrip(tmpdir: Path):
 
     # check we can get the subarray description:
     sub = SubarrayDescription.from_hdf(output_path)
-    assert sub.num_tels > 0
+    assert sub.n_tels > 0
 
     # check a few things in the output just to make sure there is output. For a
     # full test of the data model, a verify tool should be created.
     with tables.open_file(output_path) as h5file:
-        images = h5file.get_node("/dl1/event/telescope/images/tel_001")
+        images = h5file.get_node("/dl1/event/telescope/images/tel_004")
 
         assert len(images) > 0
         assert images.col("image").dtype == np.int32
@@ -204,8 +222,7 @@ def test_dl1writer_no_events(tmpdir: Path):
     """
 
     output_path = Path(tmpdir / "no_events.dl1.h5")
-    dataset = "lst_prod3_calibration_and_mcphotons.simtel.zst"
-    with EventSource(get_dataset_path(dataset), focal_length_choice='nominal') as source:
+    with EventSource("dataset://gamma_prod5.simtel.zst") as source:
         # exhaust source
         for _ in source:
             pass
@@ -244,7 +261,9 @@ def test_metadata(tmpdir: Path):
                     "name": "Maximilian Nöthe",
                     "email": "maximilian.noethe@tu-dortmund.de",
                     "organization": "TU Dortmund",
-                }
+                },
+                "Instrument": {"site": "CTA-North", "id_": "alpha"},
+                "context_metadata": {"EXAMPLE": "test_value"},
             }
         }
     )
@@ -266,6 +285,9 @@ def test_metadata(tmpdir: Path):
             assert meta["CTA CONTACT NAME"] == "Maximilian Nöthe"
             assert meta["CTA CONTACT EMAIL"] == "maximilian.noethe@tu-dortmund.de"
             assert meta["CTA CONTACT ORGANIZATION"] == "TU Dortmund"
+            assert meta["CTA INSTRUMENT SITE"] == "CTA-North"
+            assert meta["CTA INSTRUMENT ID"] == "alpha"
+            assert meta["CONTEXT EXAMPLE"] == "test_value"
 
 
 def test_write_only_r1(r1_hdf5_file):
