@@ -1,22 +1,23 @@
 import enum
+
 import numpy as np
+import pandas as pd
 import pytest
 import tables
-import pandas as pd
 from astropy import units as u
 from astropy.time import Time
 
-from ctapipe.core.container import Container, Field
 from ctapipe import containers
 from ctapipe.containers import (
+    HillasParametersContainer,
+    LeakageContainer,
     R0CameraContainer,
     SimulatedShowerContainer,
-    HillasParametersContainer,
     TelEventIndexContainer,
-    LeakageContainer,
 )
-from ctapipe.io.hdf5tableio import HDF5TableWriter, HDF5TableReader
+from ctapipe.core.container import Container, Field
 from ctapipe.io import read_table
+from ctapipe.io.hdf5tableio import HDF5TableReader, HDF5TableWriter
 
 
 @pytest.fixture(scope="session")
@@ -47,6 +48,45 @@ def test_h5_file(tmp_path_factory):
     return path
 
 
+def test_read_meta(test_h5_file):
+    """Test reading meta information"""
+    from ctapipe import __version__
+    from ctapipe.io.hdf5tableio import get_node_meta
+
+    with tables.open_file(test_h5_file, "r") as f:
+        meta = get_node_meta(f.root["/R0/tel_001"])
+
+        # check we don't have anything else
+        # system attributes and column metadata should be excluded
+        assert len(meta) == 3
+        assert meta["CTAPIPE_VERSION"] == __version__
+        assert type(meta["CTAPIPE_VERSION"]) is str
+
+        assert meta["date"] == "2020-10-10"
+        assert type(meta["date"]) is str
+
+        assert meta["test_attribute"] == 3.14159
+        assert type(meta["test_attribute"]) is float
+
+
+def test_read_column_attrs(test_h5_file):
+    """Test reading meta information"""
+    from ctapipe.io.hdf5tableio import get_column_attrs
+
+    with tables.open_file(test_h5_file, "r") as f:
+        column_attrs = get_column_attrs(f.root["/R0/sim_shower"])
+        assert len(column_attrs) == len(SimulatedShowerContainer.fields)
+        assert column_attrs["energy"]["POS"] == 0
+        assert column_attrs["energy"]["TRANSFORM"] == "quantity"
+        assert column_attrs["energy"]["UNIT"] == "TeV"
+        assert column_attrs["energy"]["DTYPE"] == np.float64
+
+        assert column_attrs["alt"]["POS"] == 1
+        assert column_attrs["alt"]["TRANSFORM"] == "quantity"
+        assert column_attrs["alt"]["UNIT"] == "deg"
+        assert column_attrs["alt"]["DTYPE"] == np.float64
+
+
 def test_append_container(tmp_path):
     path = tmp_path / "test_append.h5"
     with HDF5TableWriter(path, mode="w") as writer:
@@ -67,6 +107,13 @@ def test_append_container(tmp_path):
     assert np.all(table["event_id"] == np.tile(np.arange(10), 2))
 
 
+def test_reader_container_reuse(test_h5_file):
+    """Test the reader does not reuse the same container instance"""
+    with HDF5TableReader(test_h5_file) as reader:
+        iterator = reader.read("/R0/sim_shower", SimulatedShowerContainer)
+        assert next(iterator) is not next(iterator)
+
+
 def test_read_multiple_containers(tmp_path):
     path = tmp_path / "test_append.h5"
     hillas_parameter_container = HillasParametersContainer(
@@ -88,9 +135,7 @@ def test_read_multiple_containers(tmp_path):
 
     # test reading both containers separately
     with HDF5TableReader(path) as reader:
-        generator = reader.read(
-            "/dl1/params", HillasParametersContainer(), prefixes=True
-        )
+        generator = reader.read("/dl1/params", HillasParametersContainer, prefixes=True)
         hillas = next(generator)
     for value, read_value in zip(
         hillas_parameter_container.as_dict().values(), hillas.as_dict().values()
@@ -98,7 +143,7 @@ def test_read_multiple_containers(tmp_path):
         np.testing.assert_equal(value, read_value)
 
     with HDF5TableReader(path) as reader:
-        generator = reader.read("/dl1/params", LeakageContainer(), prefixes=True)
+        generator = reader.read("/dl1/params", LeakageContainer, prefixes=True)
         leakage = next(generator)
     for value, read_value in zip(
         leakage_container.as_dict().values(), leakage.as_dict().values()
@@ -109,7 +154,7 @@ def test_read_multiple_containers(tmp_path):
     with HDF5TableReader(path) as reader:
         generator = reader.read(
             "/dl1/params",
-            [HillasParametersContainer(), LeakageContainer()],
+            (HillasParametersContainer, LeakageContainer),
             prefixes=True,
         )
         hillas_, leakage_ = next(generator)
@@ -140,7 +185,7 @@ def test_read_without_prefixes(tmp_path):
     )
 
     with HDF5TableWriter(path, group_name="dl1", add_prefix=False) as writer:
-        writer.write("params", [hillas_parameter_container, leakage_container])
+        writer.write("params", (hillas_parameter_container, leakage_container))
 
     df = pd.read_hdf(path, key="/dl1/params")
     assert "fov_lon" in df.columns
@@ -150,7 +195,7 @@ def test_read_without_prefixes(tmp_path):
     with HDF5TableReader(path) as reader:
         generator = reader.read(
             "/dl1/params",
-            [HillasParametersContainer(), LeakageContainer()],
+            (HillasParametersContainer, LeakageContainer),
             prefixes=False,
         )
         hillas_, leakage_ = next(generator)
@@ -169,8 +214,8 @@ def test_read_without_prefixes(tmp_path):
     with HDF5TableReader(path) as reader:
         generator = reader.read(
             "/dl1/params",
-            [HillasParametersContainer(prefix=""), LeakageContainer(prefix="")],
-            prefixes=True,
+            (HillasParametersContainer, LeakageContainer),
+            prefixes=["", ""],
         )
         hillas_, leakage_ = next(generator)
 
@@ -213,7 +258,7 @@ def test_read_duplicated_container_types(tmp_path):
     with HDF5TableReader(path) as reader:
         generator = reader.read(
             "/dl1/params",
-            [HillasParametersContainer(), HillasParametersContainer()],
+            (HillasParametersContainer, HillasParametersContainer),
             prefixes=["hillas_1", "hillas_2"],
         )
         hillas_1, hillas_2 = next(generator)
@@ -241,7 +286,7 @@ def test_custom_prefix(tmp_path):
 
     with HDF5TableReader(path) as reader:
         generator = reader.read(
-            "/dl1/params", HillasParametersContainer(), prefixes="custom"
+            "/dl1/params", HillasParametersContainer, prefixes="custom"
         )
         read_container = next(generator)
     assert isinstance(read_container, HillasParametersContainer)
@@ -257,7 +302,7 @@ def test_units(tmp_path):
     class WithUnits(Container):
         inverse_length = Field(5 / u.m, "foo")
         time = Field(1 * u.s, "bar", unit=u.s)
-        grammage = Field(2 * u.g / u.cm ** 2, "baz", unit=u.g / u.cm ** 2)
+        grammage = Field(2 * u.g / u.cm**2, "baz", unit=u.g / u.cm**2)
 
     c = WithUnits()
 
@@ -265,9 +310,9 @@ def test_units(tmp_path):
         writer.write("units", c)
 
     with tables.open_file(path, "r") as f:
-        assert f.root.data.units.attrs["inverse_length_UNIT"] == "m**-1"
-        assert f.root.data.units.attrs["time_UNIT"] == "s"
-        assert f.root.data.units.attrs["grammage_UNIT"] == "cm**-2.g"
+        assert f.root.data.units.attrs["CTAFIELD_0_UNIT"] == "m**-1"
+        assert f.root.data.units.attrs["CTAFIELD_1_UNIT"] == "s"
+        assert f.root.data.units.attrs["CTAFIELD_2_UNIT"] == "cm**-2.g"
 
 
 def test_write_containers(tmp_path):
@@ -300,7 +345,7 @@ def test_write_bool(tmp_path):
 
     c = C()
     with HDF5TableReader(path) as reader:
-        c_reader = reader.read("/test/c", c)
+        c_reader = reader.read("/test/c", C)
         for i in range(2):
             cur = next(c_reader)
             expected = (i % 2) == 0
@@ -317,30 +362,26 @@ def test_write_large_integer(tmp_path):
     exps = [15, 31, 63]
     with HDF5TableWriter(path, "test") as writer:
         for exp in exps:
-            c = C(value=2 ** exp - 1)
+            c = C(value=2**exp - 1)
             writer.write("c", c)
 
     c = C()
     with HDF5TableReader(path) as reader:
-        c_reader = reader.read("/test/c", c)
+        c_reader = reader.read("/test/c", C)
         for exp in exps:
             cur = next(c_reader)
-            assert cur.value == 2 ** exp - 1
+            assert cur.value == 2**exp - 1
 
 
 def test_read_container(test_h5_file):
-    r0tel1 = R0CameraContainer()
-    r0tel2 = R0CameraContainer()
-    sim_shower = SimulatedShowerContainer()
-
     with HDF5TableReader(test_h5_file) as reader:
 
         # get the generators for each table
         # test supplying a single container as well as an
         # iterable with one entry only
-        simtab = reader.read("/R0/sim_shower", (sim_shower,))
-        r0tab1 = reader.read("/R0/tel_001", r0tel1)
-        r0tab2 = reader.read("/R0/tel_002", r0tel2)
+        simtab = reader.read("/R0/sim_shower", (SimulatedShowerContainer,))
+        r0tab1 = reader.read("/R0/tel_001", R0CameraContainer)
+        r0tab2 = reader.read("/R0/tel_002", R0CameraContainer)
 
         # read all 3 tables in sync
         for _ in range(3):
@@ -359,10 +400,9 @@ def test_read_container(test_h5_file):
 
 
 def test_read_whole_table(test_h5_file):
-    sim_shower = SimulatedShowerContainer()
 
     with HDF5TableReader(test_h5_file) as reader:
-        for cont in reader.read("/R0/sim_shower", sim_shower):
+        for cont in reader.read("/R0/sim_shower", SimulatedShowerContainer):
             print(cont)
 
 
@@ -400,13 +440,11 @@ def test_reader_closes_file(test_h5_file):
 
 def test_with_context_reader(test_h5_file):
 
-    sim_shower = SimulatedShowerContainer()
-
     with HDF5TableReader(test_h5_file) as h5_table:
 
         assert h5_table._h5file.isopen == 1
 
-        for cont in h5_table.read("/R0/sim_shower", sim_shower):
+        for cont in h5_table.read("/R0/sim_shower", SimulatedShowerContainer):
             print(cont)
 
     assert h5_table._h5file.isopen == 0
@@ -455,24 +493,24 @@ def test_append_mode(tmp_path):
     class ContainerA(Container):
         a = Field(int)
 
-    a = ContainerA(a=1)
+    container = ContainerA(a=1)
 
     # First open with 'w' mode to clear the file and add a Container
     with HDF5TableWriter(path, "group") as h5:
-        h5.write("table_1", a)
+        h5.write("table_1", container)
 
     # Try to append A again
     with HDF5TableWriter(path, "group", mode="a") as h5:
-        h5.write("table_2", a)
+        h5.write("table_2", container)
 
     # Check if file has two tables with a = 1
     with HDF5TableReader(path) as h5:
 
-        for a in h5.read("/group/table_1", ContainerA()):
-            assert a.a == 1
+        for container in h5.read("/group/table_1", ContainerA):
+            assert container.a == 1
 
-        for a in h5.read("/group/table_2", ContainerA()):
-            assert a.a == 1
+        for container in h5.read("/group/table_2", ContainerA):
+            assert container.a == 1
 
 
 def test_write_to_any_location(tmp_path):
@@ -482,20 +520,20 @@ def test_write_to_any_location(tmp_path):
     class ContainerA(Container):
         a = Field(0, "some integer field")
 
-    a = ContainerA(a=1)
+    container = ContainerA(a=1)
 
     with HDF5TableWriter(path, group_name=loc + "/group_1") as h5:
         for _ in range(5):
-            h5.write("table", a)
-            h5.write("deeper/table2", a)
+            h5.write("table", container)
+            h5.write("deeper/table2", container)
 
     with HDF5TableReader(path) as h5:
-        for a in h5.read("/" + loc + "/group_1/table", ContainerA()):
-            assert a.a == 1
+        for container in h5.read(f"/{loc}/group_1/table", ContainerA):
+            assert container.a == 1
 
     with HDF5TableReader(path) as h5:
-        for a in h5.read("/" + loc + "/group_1/deeper/table2", ContainerA()):
-            assert a.a == 1
+        for container in h5.read(f"/{loc}/group_1/deeper/table2", ContainerA):
+            assert container.a == 1
 
 
 class WithNormalEnum(Container):
@@ -532,7 +570,7 @@ def test_read_write_container_with_enum(tmp_path):
     with HDF5TableReader(tmp_file, mode="r") as h5_table:
         for group_name in ["data/"]:
             group_name = "/{}table".format(group_name)
-            for data in h5_table.read(group_name, WithNormalEnum()):
+            for data in h5_table.read(group_name, WithNormalEnum):
                 assert isinstance(data.event_type, WithNormalEnum.EventType)
 
 
@@ -564,7 +602,7 @@ def test_read_write_container_with_int_enum(tmp_path):
     with HDF5TableReader(tmp_file, mode="r") as h5_table:
         for group_name in ["data/"]:
             group_name = "/{}table".format(group_name)
-            for data in h5_table.read(group_name, WithIntEnum()):
+            for data in h5_table.read(group_name, WithIntEnum):
                 assert isinstance(data.event_type, WithIntEnum.EventType)
 
 
@@ -573,7 +611,7 @@ def test_column_exclusions(tmp_path):
     tmp_file = tmp_path / "test_column_exclusions.hdf5"
 
     class SomeContainer(Container):
-        container_prefix = ""
+        default_prefix = ""
         hillas_x = Field(None)
         hillas_y = Field(None)
         impact_x = Field(None)
@@ -596,13 +634,13 @@ def test_column_exclusions(tmp_path):
     # check that we get back the transformed values (note here a round trip will
     # not work, as there is no inverse transform in this test)
     with HDF5TableReader(tmp_file, mode="r") as reader:
-        data = next(reader.read("/data/mytable", SomeContainer()))
+        data = next(reader.read("/data/mytable", SomeContainer))
         assert data.hillas_x is None
         assert data.hillas_y is None
         assert data.impact_x == 15
         assert data.impact_y == 15
 
-        data = next(reader.read("/data/anothertable", SomeContainer()))
+        data = next(reader.read("/data/anothertable", SomeContainer))
         assert data.hillas_x is None
         assert data.hillas_y is None
         assert data.impact_x is None
@@ -610,14 +648,14 @@ def test_column_exclusions(tmp_path):
 
 
 def test_column_transforms(tmp_path):
-    """ ensure a user-added column transform is applied """
+    """ensure a user-added column transform is applied"""
     from ctapipe.containers import NAN_TIME
     from ctapipe.io.tableio import FixedPointColumnTransform
 
     tmp_file = tmp_path / "test_column_transforms.hdf5"
 
     class SomeContainer(Container):
-        container_prefix = ""
+        default_prefix = ""
 
         current = Field(1 * u.A, unit=u.uA)
         time = Field(NAN_TIME)
@@ -634,7 +672,7 @@ def test_column_transforms(tmp_path):
 
     # check that we get a length-3 array when reading back
     with HDF5TableReader(tmp_file, mode="r") as reader:
-        data = next(reader.read("/data/mytable", SomeContainer()))
+        data = next(reader.read("/data/mytable", SomeContainer))
         assert data.current.value == 1e6
         assert data.current.unit == u.uA
         assert isinstance(data.time, Time)
@@ -644,13 +682,13 @@ def test_column_transforms(tmp_path):
 
 
 def test_fixed_point_column_transform(tmp_path):
-    """ ensure a user-added column transform is applied """
+    """ensure a user-added column transform is applied"""
     from ctapipe.io.tableio import FixedPointColumnTransform
 
     tmp_file = tmp_path / "test_column_transforms.hdf5"
 
     class SomeContainer(Container):
-        container_prefix = ""
+        default_prefix = ""
         image = Field(np.array([np.nan, np.inf, -np.inf]))
 
     cont = SomeContainer()
@@ -669,8 +707,8 @@ def test_fixed_point_column_transform(tmp_path):
         writer.write("unsigned", cont)
 
     with HDF5TableReader(tmp_file, mode="r") as reader:
-        signed = next(reader.read("/data/signed", SomeContainer()))
-        unsigned = next(reader.read("/data/unsigned", SomeContainer()))
+        signed = next(reader.read("/data/signed", SomeContainer))
+        unsigned = next(reader.read("/data/unsigned", SomeContainer))
 
         for data in (signed, unsigned):
             # check we get our original nans back
@@ -680,7 +718,7 @@ def test_fixed_point_column_transform(tmp_path):
 
 
 def test_column_transforms_regexps(tmp_path):
-    """ ensure a user-added column transform is applied when given as a regexp"""
+    """ensure a user-added column transform is applied when given as a regexp"""
 
     tmp_file = tmp_path / "test_column_transforms.hdf5"
 
@@ -688,7 +726,7 @@ def test_column_transforms_regexps(tmp_path):
         return x * 10
 
     class SomeContainer(Container):
-        container_prefix = ""
+        default_prefix = ""
         hillas_x = Field(1)
         hillas_y = Field(1)
 
@@ -704,11 +742,11 @@ def test_column_transforms_regexps(tmp_path):
     # check that we get back the transformed values (note here a round trip will
     # not work, as there is no inverse transform in this test)
     with HDF5TableReader(tmp_file, mode="r") as reader:
-        data = next(reader.read("/data/mytable", SomeContainer()))
+        data = next(reader.read("/data/mytable", SomeContainer))
         assert data.hillas_x == 10
         assert data.hillas_y == 10
 
-        data = next(reader.read("/data/anothertable", SomeContainer()))
+        data = next(reader.read("/data/anothertable", SomeContainer))
         assert data.hillas_x == 1
         assert data.hillas_y == 10
 
@@ -727,7 +765,7 @@ def test_time(tmp_path):
         writer.write("table", container)
 
     with HDF5TableReader(tmp_file, mode="r") as reader:
-        for data in reader.read("/data/table", TimeContainer()):
+        for data in reader.read("/data/table", TimeContainer):
             assert isinstance(data.time, Time)
             assert data.time.scale == "tai"
             assert data.time.format == "mjd"
@@ -765,11 +803,12 @@ def test_filters(tmp_path):
 
 
 def test_column_order_single_container(tmp_path):
-    """ Test that columns are written in the order the containers define them"""
+    """Test that columns are written in the order the containers define them"""
     path = tmp_path / "test.h5"
 
     class Container1(Container):
         b = Field(1, "b")
+        c = Field("foo", "a", type=str, max_length=20)
         a = Field(2, "a")
 
     # test with single container
@@ -778,11 +817,11 @@ def test_column_order_single_container(tmp_path):
         writer.write("foo", c)
 
     with tables.open_file(path, "r") as f:
-        assert f.root.foo[:].dtype.names == ("b", "a")
+        assert f.root.foo[:].dtype.names == ("b", "c", "a")
 
 
 def test_column_order_multiple_containers(tmp_path):
-    """ Test that columns are written in the order the containers define them"""
+    """Test that columns are written in the order the containers define them"""
     path = tmp_path / "test.h5"
 
     class Container1(Container):
@@ -849,7 +888,7 @@ def test_strings(tmp_path):
 
     # when not giving a max_len, should be taken from the first container
     class Container1(Container):
-        container_prefix = ""
+        default_prefix = ""
         string = Field("", "test string")
 
     path = tmp_path / "test.h5"
@@ -866,7 +905,7 @@ def test_strings(tmp_path):
     assert table["string"].tolist() == ["Hello", "öä"]
 
     class Container2(Container):
-        container_prefix = ""
+        default_prefix = ""
         string = Field("", "test string", max_length=10)
 
     path = tmp_path / "test.h5"
@@ -885,7 +924,92 @@ def test_strings(tmp_path):
 
     # test this also works with table reader
     with HDF5TableReader(path) as reader:
-        generator = reader.read("/strings", Container2())
+        generator = reader.read("/strings", Container2)
         for string in expected:
             c = next(generator)
             assert c.string == string
+
+
+def test_prefix_in_output_container(tmp_path):
+    """Test that output containers retain the used prefix"""
+
+    class Container1(Container):
+        default_prefix = ""
+        value = Field(-1, "value")
+
+    path = tmp_path / "prefix.h5"
+    with HDF5TableWriter(path, mode="w", add_prefix=True) as writer:
+        for value in (1, 2, 3):
+            writer.write("values", Container1(value=value, prefix="custom_prefix"))
+
+    with HDF5TableReader(path) as reader:
+        generator = reader.read("/values", Container1, prefixes="custom_prefix")
+
+        for value in (1, 2, 3):
+            c = next(generator)
+            assert c.prefix == "custom_prefix"
+            assert c.value == value
+
+
+def test_can_read_without_prefix_given(tmp_path):
+    """Test that output containers retain the used prefix"""
+
+    class Container1(Container):
+        default_prefix = ""
+        value = Field(-1, "value")
+
+    path = tmp_path / "prefix.h5"
+    with HDF5TableWriter(path, mode="w", add_prefix=True) as writer:
+        for value in (1, 2, 3):
+            writer.write("values", Container1(value=value, prefix="custom_prefix"))
+
+    # test we can read back the data without knowing the "custom_prefix"
+    with HDF5TableReader(path) as reader:
+        generator = reader.read("/values", Container1)
+
+        for value in (1, 2, 3):
+            c = next(generator)
+            assert c.value == value
+            assert c.prefix == "custom_prefix"
+
+
+def test_can_read_same_containers(tmp_path):
+    """Test we can read two identical containers with different prefixes"""
+
+    class Container1(Container):
+        default_prefix = ""
+        value = Field(-1, "value")
+
+    # test with two of the same container with different prefixes
+    path = tmp_path / "two_containers.h5"
+    with HDF5TableWriter(path, mode="w", add_prefix=True) as writer:
+        for value in (1, 2, 3):
+            writer.write(
+                "values",
+                [
+                    Container1(value=value, prefix="foo"),
+                    Container1(value=5 * value, prefix="bar"),
+                ],
+            )
+
+    # This needs to fail since the mapping is not unique
+    with HDF5TableReader(path) as reader:
+        with pytest.raises(IOError):
+            generator = reader.read("/values", [Container1, Container1])
+            next(generator)
+
+    # But when explicitly giving the prefixes, this works and order
+    # should not be important
+    reader = HDF5TableReader(path)
+    generator = reader.read(
+        "/values",
+        [Container1, Container1],
+        prefixes=["bar", "foo"],
+    )
+
+    for value in (1, 2, 3):
+        c1, c2 = next(generator)
+        assert c1.value == 5 * value
+        assert c1.prefix == "bar"
+        assert c2.value == value
+        assert c2.prefix == "foo"
