@@ -5,6 +5,7 @@ import numpy as np
 
 from ctapipe.core import Tool
 from ctapipe.core.traits import Int, IntTelescopeParameter, Path
+from ctapipe.exceptions import TooFewEvents
 from ctapipe.io import TableLoader
 from ctapipe.reco import CrossValidator, EnergyRegressor
 from ctapipe.reco.preprocessing import check_valid_rows
@@ -113,11 +114,19 @@ class TrainEnergyRegressor(Tool):
 
     def _read_table(self, telescope_type):
         table = self.loader.read_telescope_events([telescope_type])
-
         self.log.info("Events read from input: %d", len(table))
+        if len(table) == 0:
+            raise TooFewEvents(
+                f"Input file does not contain any events for telescope type {telescope_type}"
+            )
+
         mask = self.regressor.quality_query.get_table_mask(table)
         table = table[mask]
         self.log.info("Events after applying quality query: %d", len(table))
+        if len(table) == 0:
+            raise TooFewEvents(
+                f"No events after quality query for telescope type {telescope_type}"
+            )
 
         table = self.regressor.feature_generator(table)
 
